"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.HighlightLine = void 0;

var _react = require("devtools/client/shared/vendor/react");

var _reactPropTypes = _interopRequireDefault(require("devtools/client/shared/vendor/react-prop-types"));

loader.lazyRequireGetter(this, "_index", "devtools/client/debugger/src/utils/editor/index");
loader.lazyRequireGetter(this, "_sourceDocuments", "devtools/client/debugger/src/utils/editor/source-documents");

var _reactRedux = require("devtools/client/shared/vendor/react-redux");

loader.lazyRequireGetter(this, "_index2", "devtools/client/debugger/src/selectors/index");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function isDebugLine(selectedFrame, selectedLocation) {
  if (!selectedFrame) {
    return false;
  }

  return selectedFrame.location.source.id == selectedLocation.source.id && selectedFrame.location.line == selectedLocation.line;
}

function isDocumentReady(selectedLocation, selectedSourceTextContent) {
  return selectedLocation && selectedSourceTextContent && (0, _sourceDocuments.hasDocument)(selectedLocation.source.id);
}

class HighlightLine extends _react.Component {
  constructor(...args) {
    super(...args);

    _defineProperty(this, "isStepping", false);

    _defineProperty(this, "previousEditorLine", null);
  }

  static get propTypes() {
    return {
      pauseCommand: _reactPropTypes.default.oneOf(["expression", "resume", "stepOver", "stepIn", "stepOut"]),
      selectedFrame: _reactPropTypes.default.object,
      selectedLocation: _reactPropTypes.default.object.isRequired,
      selectedSourceTextContent: _reactPropTypes.default.object.isRequired
    };
  }

  shouldComponentUpdate(nextProps) {
    const {
      selectedLocation,
      selectedSourceTextContent
    } = nextProps;
    return this.shouldSetHighlightLine(selectedLocation, selectedSourceTextContent);
  }

  componentDidUpdate(prevProps) {
    this.completeHighlightLine(prevProps);
  }

  componentDidMount() {
    this.completeHighlightLine(null);
  }

  shouldSetHighlightLine(selectedLocation, selectedSourceTextContent) {
    const {
      line
    } = selectedLocation;
    const editorLine = (0, _index.toEditorLine)(selectedLocation.source.id, line);

    if (!isDocumentReady(selectedLocation, selectedSourceTextContent)) {
      return false;
    }

    if (this.isStepping && editorLine === this.previousEditorLine) {
      return false;
    }

    return true;
  }

  completeHighlightLine(prevProps) {
    const {
      pauseCommand,
      selectedLocation,
      selectedFrame,
      selectedSourceTextContent,
      shouldHighlightSelectedLocation
    } = this.props;

    if (pauseCommand) {
      this.isStepping = true;
    }

    (0, _index.startOperation)();

    if (prevProps) {
      this.clearHighlightLine(prevProps.selectedLocation, prevProps.selectedSourceTextContent);
    }

    if (shouldHighlightSelectedLocation) {
      this.setHighlightLine(selectedLocation, selectedFrame, selectedSourceTextContent);
    }

    (0, _index.endOperation)();
  }

  setHighlightLine(selectedLocation, selectedFrame, selectedSourceTextContent) {
    const {
      line
    } = selectedLocation;

    if (!this.shouldSetHighlightLine(selectedLocation, selectedSourceTextContent)) {
      return;
    }

    this.isStepping = false;
    const sourceId = selectedLocation.source.id;
    const editorLine = (0, _index.toEditorLine)(sourceId, line);
    this.previousEditorLine = editorLine;

    if (!line || isDebugLine(selectedFrame, selectedLocation)) {
      return;
    }

    const doc = (0, _sourceDocuments.getDocument)(sourceId);
    doc.addLineClass(editorLine, "wrap", "highlight-line");
    this.resetHighlightLine(doc, editorLine);
  }

  resetHighlightLine(doc, editorLine) {
    const editorWrapper = document.querySelector(".editor-wrapper");

    if (editorWrapper === null) {
      return;
    }

    const duration = parseInt(getComputedStyle(editorWrapper).getPropertyValue("--highlight-line-duration"), 10);
    setTimeout(() => doc && doc.removeLineClass(editorLine, "wrap", "highlight-line"), duration);
  }

  clearHighlightLine(selectedLocation, selectedSourceTextContent) {
    if (!isDocumentReady(selectedLocation, selectedSourceTextContent)) {
      return;
    }

    const {
      line
    } = selectedLocation;
    const sourceId = selectedLocation.source.id;
    const editorLine = (0, _index.toEditorLine)(sourceId, line);
    const doc = (0, _sourceDocuments.getDocument)(sourceId);
    doc.removeLineClass(editorLine, "wrap", "highlight-line");
  }

  render() {
    return null;
  }

}

exports.HighlightLine = HighlightLine;

var _default = (0, _reactRedux.connect)(state => {
  const selectedLocation = (0, _index2.getSelectedLocation)(state);

  if (!selectedLocation) {
    throw new Error("must have selected location");
  }

  return {
    pauseCommand: (0, _index2.getPauseCommand)(state, (0, _index2.getCurrentThread)(state)),
    shouldHighlightSelectedLocation: (0, _index2.getShouldHighlightSelectedLocation)(state),
    selectedFrame: (0, _index2.getVisibleSelectedFrame)(state),
    selectedLocation,
    selectedSourceTextContent: (0, _index2.getSelectedSourceTextContent)(state)
  };
})(HighlightLine);

exports.default = _default;